from bs4 import BeautifulSoup
import requests
import pandas as pd
import numpy as np


def get_ebay_url(prompt):
    ebay_url = 'https://www.ebay.com/sch/i.html?_from=R40&_trksid=p2510209.m570.l1312&_nkw=%s&_sacat=0' % prompt
    return ebay_url


def get_listings(ebay_url):
    r = requests.get(ebay_url).text
    soup = BeautifulSoup(r, features="lxml")

    data = soup.find('ul', {'class': 'srp-results srp-list clearfix'})

    listings = data.find_all('li')

    return listings


def iterate_listings(listings):
    names, prices, links = [], [], []

    for listing in listings:
        if listing.find('h3', {'class': 's-item__title'}) == None:
            continue

        for name in listing.find('h3', {'class': 's-item__title'}):
            if name.text == 'New Listing':
                continue
            names.append(name.text)

        for link in listing.find_all('a', {'class': 's-item__link'}):
            links.append(link['href'])

        temp_price = ''
        for price in listing.find('span', {'class', 's-item__price'}):
            temp_price += price.text
        prices.append(temp_price)

    return names, prices, links


def loop_to_clean_prices(price):
    if 'to' in price:
        price = price.split(' ')[0].replace('$', '')
        try:
            return float(price)
        except ValueError:
            return np.NaN

    else:
        try:
            return float(price.replace('$', ''))
        except ValueError:
            return np.NaN


def clean_prices(prices):
    prices = [loop_to_clean_prices(price) for price in prices]
    return prices


def column_previous_prices(prices):
    previous_prices = ['new entry' for i in range(len(prices))]

    return previous_prices


def column_status(prices):
    status = ['new' for i in range(len(prices))]

    return status


def create_item_data(status, names, prices, previous_prices, links):
    item_data = list(zip(status, names, prices, previous_prices, links))

    return item_data


def make_df(item_data):
    df = pd.DataFrame(item_data, columns=['status', 'name', 'price', 'previous_price', 'link'])

    return df


def get_data_in_df_format(prompt):

    ebay_url = get_ebay_url(prompt)
    listings = get_listings(ebay_url)
    names, prices, links = iterate_listings(listings)

    prices = clean_prices(prices)
    previous_prices = column_previous_prices(prices)
    status = column_status(prices)

    item_data = create_item_data(status, names, prices, previous_prices, links)

    df = make_df(item_data)

    return df



import time
from random import choice
import numpy as np


def calculate_total_wait_time(down_time_hours):
    down_time_seconds = down_time_hours * 60 * 60

    variance_in_seconds = np.arange(-600, 660, 60)
    seconds_added_or_subtracted = choice(variance_in_seconds)

    down_time_seconds += seconds_added_or_subtracted

    return down_time_seconds


def sleep_time(down_time_seconds):
    time.sleep(down_time_seconds)



from user_prompt import *
from scrape_get_data import *
from email_module import *
from misc_module import *
from wait_module import *
from compare_old_and_new_entry import *


def tracker():
    prompt = prompt_user_for_item()
    df = get_data_in_df_format(prompt)

    down_time_hours = ask_wait_time()

    req_price = ask_requested_price()
    req_df = filter_data_for_user(df, req_price)
    user_email = ask_email()

    main_email_module(req_df, user_email, prompt)

    save_df(df)

    while True:
        down_time_seconds = calculate_total_wait_time(down_time_hours)
        sleep_time(down_time_seconds)

        df = get_data_in_df_format(prompt)
        original_df = get_original_df()

        updated_df = compare_old_new_entry_in_df_format(df, original_df)

        req_df = filter_data_for_user(updated_df, req_price)
        save_df(updated_df)
        main_email_module(req_df, user_email, prompt)



def prompt_user_for_item():
    prompt = input('What item do you want to track? ')
    prompt = prompt.replace(' ', '+')
    return prompt


def ask_wait_time():
    while True:
        try:
            down_time_hours = float(input('How much time do you want between updates in hours? '))
            return down_time_hours
        except ValueError:
            print('Insert a valid hour')


def ask_requested_price():
    while True:
        req_price = input(
            'What is a range of prices you would like to search the product for? input example: "200,250" ')
        req_price = req_price.split(',')

        if len(req_price) != 2:
            print('Insert a valid range')
            continue

        req_price = [float_error_price_range(price) for price in req_price]

        if False in req_price:
            print('Insert a valid range')
            continue

        if req_price[0] > req_price[1]:
            print('Insert a valid range')
            continue

        return req_price


def float_error_price_range(price):
    try:
        return float(price)
    except ValueError:
        return False


def ask_email():
    while True:
        user_email = input('Insert an email address to receive updates about your item: ')
        if '.' not in user_email[-6:] or '@' not in user_email:
            print('Insert a valid Email Address')
            continue

        return user_email



def check_entries(df, old_df):
    updated_entries = df[df['name'].isin(old_df['name'])].drop_duplicates(subset=['name'])

    new_entries = df[~df['name'].isin(old_df['name'])]

    return updated_entries, new_entries


def update_entries_price_in_original_df(updated_entries, old_df):
    updated_entries = updated_entries.sort_values('name').reset_index()

    temporary_df = old_df[old_df['name'].isin(updated_entries['name'])].sort_values('name').drop_duplicates(
        subset=['name']).reset_index()

    updated_entries.loc[(updated_entries.previous_price == 'new entry'), 'previous_price'] = 'no change in price'

    price_mask = temporary_df['price'] != updated_entries['price']
    updated_entries.loc[price_mask, 'previous_price'] = temporary_df.loc[price_mask, 'price']

    temporary_df[temporary_df.columns[1:6]] = updated_entries[updated_entries.columns[1:6]]

    temporary_df = temporary_df.set_index('index')

    old_df.loc[temporary_df.index] = temporary_df

    old_df.loc[(old_df.status == 'new'), 'status'] = 'old'
    old_df.loc[(old_df.previous_price == 'new entry'), 'previous_price'] = 'was not seen'

    return old_df


def append_new_entries(updated_df, new_entries):
    updated_df = updated_df.append(new_entries, ignore_index=True)

    return updated_df


def compare_old_new_entry_in_df_format(df, old_df):
    updated_entries, new_entries = check_entries(df, old_df)

    updated_df = update_entries_price_in_original_df(updated_entries, old_df)

    updated_df = append_new_entries(updated_df, new_entries)

    return updated_df



from setuptools import setup

with open("README.md", "r") as fh:
	long_description = fh.read()

setup(
	name = 'eBay email tracker',
	version = '0.0.1',
	description = 'Scapes an entire search page of a particular item on eBay and sends regular updates to an email address',
	py_modules = ['ebay_email_tracker', 'compare_old_and_new_entry', 'email_module', 'scrape_get_data', 'user_prompt', 'wait_module',' misc_module'],
	package_dir = {'': 'src'},
	classifiers = [
		"Programming Language :: Python :: 3",
		"Programming Language :: Python :: 3.9",
		"Programming Language :: Python :: 3.10",
		],
	long_description=long_description,
	long_description_content_type="text/markdown",
	install_requires = [
	"pandas~=1.3.5",
	"numpy~=1.22.0",
	"requests~=2.27.1",
	"beautifulsoup4~=4.10.0",
	"lxml~=4.7.1"],

	extras_require = {
	"dev": [
	"pytest>=3.7",]
	},

)


from os import environ
import smtplib
from email.message import EmailMessage
import functools


def set_environ_variables():
    global EMAIL
    global PASSWORD

    EMAIL = environ.get('EBAY_TRACKER_EMAIL')
    PASSWORD = environ.get('EBAY_TRACKER_PASSWORD')


def set_html_format():
    hyperlink_format = '<a href="{link}">{text}</a> <i>${price}</i>'
    link_text = functools.partial(hyperlink_format.format)
    return link_text


def build_content_email(df, link_text):
    message = '<h2><p>Here is a list of items that meet your price range</h2></p>'
    for i in range(len(df)):
        message += f"{link_text(link=df['link'].loc[i], text=df['name'].loc[i], price=df['price'].loc[i])}<br>"

    return message


def send_email(user_email, prompt, message):
    msg = EmailMessage()
    msg['Subject'] = 'eBay tracker update for: %s' % prompt
    msg['From'] = EMAIL
    msg['To'] = user_email

    msg.add_alternative(message, subtype='html')

    with smtplib.SMTP_SSL('smtp.gmail.com', 465) as smtp:
        smtp.login(EMAIL, PASSWORD)
        smtp.send_message(msg)


def main_email_module(df, user_email, prompt):
    set_environ_variables()
    link_text = set_html_format()
    message = build_content_email(df, link_text)
    send_email(user_email, prompt, message)
